@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.1.1.1 2005/02/14 14:52:21 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.2 2005/06/13 13:44:09 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -835,11 +835,16 @@ static RETSIGTYPE die(int sig)
 {
 	exit_request = 1;
 
+	pool_debug("child receives shutdown request signal %d", sig);
+
 	switch (sig)
 	{
 		case SIGTERM:	/* smart shutdown */
 			if (idle == 0)
+			{
+				pool_debug("child receives smart shutdown request but it's not in idle state");
 				return;
+			}
 
 		case SIGINT:	/* fast shutdown */
 		case SIGQUIT:	/* immediate shutdown */
