@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_auth.c,v 1.1.1.1 2005/02/14 14:52:21 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_auth.c,v 1.4 2005/06/25 00:58:46 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -109,6 +109,8 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 		return -1;
 	}
 
+	pid = ntohl(pid);
+
 	if (DUAL_MODE)
 	{
 		status = pool_read(SECONDARY(cp), &pid1, sizeof(pid1));
@@ -118,9 +120,15 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 			pool_error("pool_do_auth: read authentication kind from secondary failed");
 			return -1;
 		}
-	}
 
-	pid = ntohl(pid);
+		pid1 = ntohl(pid1);
+
+		if (pid != pid1)
+		{
+			pool_error("pool_do_auth: authentication kind does not match between master(%d) and secondary(%d)", pid, pid1);
+			return -1;
+		}
+	}
 
 	/* trust? */
 	if (pid == 0)
@@ -205,7 +213,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 			SimpleForwardToFrontend(kind, frontend, cp);
 		return -1;
 	}
-	else if (kind != 'K')
+	else if (kind != 'K')		/* BackendKeyData? */
 	{
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
@@ -887,6 +895,59 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 	return length;
 }
 
+/*
+ * read message length2 (V3 only)
+ * unlike pool_read_message_length, this returns an array of message length.
+ * the array is in the static storage, thus it will be destroyed by subsequent calls.
+ */
+int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
+{
+	int status;
+	int length, length1;
+	static int length_array[MAX_CONNECTION_SLOTS];
+
+	status = pool_read(MASTER(cp), &length, sizeof(length));
+	if (status < 0)
+	{
+		pool_error("read_message_length2: error while reading message length");
+		return NULL;
+	}
+	length = ntohl(length);
+
+	pool_debug("read_message_length2: master lenghth: %d", length);
+
+	if (DUAL_MODE)
+	{
+		status = pool_read(SECONDARY(cp), &length1, sizeof(length1));
+		if (status < 0)
+		{
+			pool_error("read_message_length2: error while reading message length from secondary backend");
+			return NULL;
+		}
+		length1 = ntohl(length1);
+
+		if (length != length1)
+		{
+			pool_debug("read_message_length2: length does not match between backends master(%d) secondary(%d)",
+					   length, length1);
+		}
+	}
+	else
+	{
+		length1 = 0;
+	}
+
+	if (length < 0 || length1 < 0)
+	{
+		pool_error("read_message_length2: invalid message length (%d) length1 (%d)", length, length1);
+		return NULL;
+	}
+
+	length_array[0] = length;
+	length_array[1] = length1;
+	return &length_array[0];
+}
+
 signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 {
 	int status;
@@ -895,7 +956,7 @@ signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 	status = pool_read(MASTER(cp), &kind, sizeof(kind));
 	if (status < 0)
 	{
-		pool_error("read_message_kind: error while reading message kind");
+		pool_error("read_kind: error while reading message kind");
 		return -1;
 	}
 
@@ -904,13 +965,13 @@ signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 		status = pool_read(SECONDARY(cp), &kind1, sizeof(kind1));
 		if (status < 0)
 		{
-			pool_error("read_message_kind: error while reading message kind from secondary backend");
+			pool_error("read_kind: error while reading message kind from secondary backend");
 			return -1;
 		}
 
 		if (kind != kind1)
 		{
-			pool_error("read_message_kind: kind does not match between backends master(%d) secondary(%d)",
+			pool_error("read_kind: kind does not match between backends master(%d) secondary(%d)",
 					   kind, kind1);
 			return -1;
 		}
