@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.3 2005/03/12 13:01:01 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.5 2005/06/25 00:58:46 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -35,6 +35,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <netinet/in.h>
+#include <ctype.h>
 
 #include "pool.h"
 
@@ -95,12 +96,20 @@ static int reset_backend(POOL_CONNECTION_POOL *backend, int qcnt);
 static int load_balance_enabled(POOL_CONNECTION_POOL *backend, char *sql);
 static void start_load_balance(POOL_CONNECTION_POOL *backend);
 static void end_load_balance(POOL_CONNECTION_POOL *backend);
+static POOL_STATUS do_command(POOL_CONNECTION *backend, char *query, int protoMajor, int no_ready_for_query);
+static int need_insert_lock(POOL_CONNECTION_POOL *backend, char *query);
+static POOL_STATUS insert_lock(POOL_CONNECTION_POOL *backend, char *query);
+static char *get_insert_command_table_name(char *query);
+static char *skip_comment(char *query);
 
 static POOL_CONNECTION_POOL_SLOT *slots[MAX_CONNECTION_SLOTS];
-static int in_load_balance;
-static int master_slave_dml;
-static int replication_was_enabled;
-static int master_slave_was_enabled;
+
+static int in_load_balance;		/* non 0 if in load balance mode */
+static int master_slave_dml;	/* non 0 if master/slave mode is specified in config file */
+static int replication_was_enabled;		/* replication mode was enabled */
+static int master_slave_was_enabled;	/* master/slave mode was enabled */
+static int internal_transaction_started;		/* to issue table lock command a transaction
+												   has been started internally */
 
 POOL_STATUS pool_process_query(POOL_CONNECTION *frontend, 
 							   POOL_CONNECTION_POOL *backend,
@@ -368,6 +377,14 @@ POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 		 * Prrocess backend Response
 		 */
 
+		if (kind == 0)
+		{
+			pool_error("kind is 0!");
+			return POOL_ERROR;
+		}
+
+		pool_debug("pool_process_query: kind from backend: %c", kind);
+
 		if (MAJOR(backend) == PROTO_MAJOR_V3)
 		{
 			switch (kind)
@@ -530,6 +547,21 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		master_slave_dml = 1;
 	}
 
+	/*
+	 * judge if we need to lock the table
+	 * to keep SERIAL data consistency among servers
+	 * conditions:
+	 * - protocol is V3
+	 * - statement is INSERT
+	 * - either "INSERT LOCK" comment exists or insert_lock directive specified
+	 */
+	if (REPLICATION && need_insert_lock(backend, string))
+	{
+		/* start a transaction if needed and lock the table */
+		if (insert_lock(backend, string) != POOL_CONTINUE)
+			return POOL_END;
+	}
+
 	/* forward the query to the backend */
 	pool_write(MASTER(backend), "Q", 1);
 
@@ -557,11 +589,12 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 	{
 		/*
 		 * in "strict mode" we need to wait for master completing the query.
-		 * note that this is applied to non SELECT queries only.
+		 * note that this is not applied if "NO STRICT" is specified as a comment.
 		 */
-		if (strncasecmp("SELECT", string, 6) &&
-			(pool_config.replication_strict || STRICT_MODE(string)))
+		if ((pool_config.replication_strict && !NO_STRICT_MODE(string)) ||
+			STRICT_MODE(string))
 		{
+			pool_debug("waiting for master completing the query");
 			if (synchronize(MASTER(backend)))
 				return POOL_END;
 		}
@@ -599,6 +632,38 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend, 
 								 POOL_CONNECTION_POOL *backend, int send_ready)
 {
+	/* if a transaction is started for insert lock, we need to close it. */
+	if (internal_transaction_started)
+	{
+		int i;
+		int len;
+		signed char state;
+
+		if ((len = pool_read_message_length(backend)) < 0)
+			return POOL_END;
+
+		pool_debug("ReadyForQuery: message length: %d", len);
+
+		len = htonl(len);
+
+		state = pool_read_kind(backend);
+		if (state < 0)
+			return POOL_END;
+
+		/* set transaction state */
+		pool_debug("ReadyForQuery: transaction state: %c", state);
+		MASTER(backend)->tstate = state;
+		if (REPLICATION)
+			SECONDARY(backend)->tstate = state;
+
+		for (i = 0;i < backend->num;i++)
+		{
+			if (do_command(backend->slots[i]->con, "COMMIT", PROTO_MAJOR_V3, 1) != POOL_CONTINUE)
+				return POOL_ERROR;
+		}
+		internal_transaction_started = 0;
+	}
+
 	pool_flush(frontend);
 
 	if (send_ready)
@@ -1568,6 +1633,7 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 	short nrows;
 	int size;
 	int hsize;
+	int slen;
 
 	i = 0;
 
@@ -1714,6 +1780,11 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 	status[i].desc = "health check user";
 	i++;
 
+	status[i].name = "insert_lock";
+	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.insert_lock);
+	status[i].desc = "insert lock";
+	i++;
+
 	status[i].name = "current_backend_host_name";
 	snprintf(status[i].value, MAXVALLEN, "%s", pool_config.current_backend_host_name);
 	status[i].desc = "current master host name";
@@ -1739,6 +1810,44 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 	status[i].desc = "number of queries in reset_query_list";
 	i++;
 
+	status[i].name = "server_status";
+
+	if (pool_config.server_status[0] == 0)
+	{
+		snprintf(status[i].value, MAXVALLEN, "master(%s on %d) unused ",
+		  pool_config.backend_host_name, pool_config.backend_port);
+	}
+	else if (pool_config.server_status[0] == 1)
+	{
+		snprintf(status[i].value, MAXVALLEN, "master(%s on %d) up ",
+		  pool_config.backend_host_name, pool_config.backend_port);
+	}
+	else if (pool_config.server_status[0] == 2)
+	{
+		snprintf(status[i].value, MAXVALLEN, "master(%s on %d) down ",
+		  pool_config.backend_host_name, pool_config.backend_port);
+	}
+
+	slen = strlen(status[i].value);
+
+	if (pool_config.server_status[1] == 0)
+	{
+		snprintf(status[i].value+slen, MAXVALLEN-slen, "secondary(%s on %d) unused",
+		  pool_config.secondary_backend_host_name, pool_config.secondary_backend_port);
+	}
+	else if (pool_config.server_status[1] == 1)
+	{
+		snprintf(status[i].value+slen, MAXVALLEN-slen, "secondary(%s on %d) up",
+		  pool_config.secondary_backend_host_name, pool_config.secondary_backend_port);
+	}
+	else if (pool_config.server_status[1] == 2)
+	{
+		snprintf(status[i].value+slen, MAXVALLEN-slen, "secondary(%s on %d) down",
+		  pool_config.secondary_backend_host_name, pool_config.secondary_backend_port);
+	}
+	status[i].desc = "server status";
+	i++;
+
 	nrows = i;
 
 	if (MAJOR(backend) == PROTO_MAJOR_V2)
@@ -2005,6 +2114,7 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 POOL_STATUS ParameterStatus(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 {
 	int len;
+	int *len_array;
 	int sendlen;
 	char *p;
 	char *name;
@@ -2012,12 +2122,14 @@ POOL_STATUS ParameterStatus(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *bac
 
 	pool_write(frontend, "S", 1);
 
-	len = pool_read_message_length(backend);
-	if (len < 0)
+	len_array = pool_read_message_length2(backend);
+
+	if (len_array == NULL)
 	{
 		return POOL_END;
 	}
 
+	len = len_array[0];
 	sendlen = htonl(len);
 	pool_write(frontend, &sendlen, sizeof(sendlen));
 
@@ -2039,9 +2151,18 @@ POOL_STATUS ParameterStatus(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *bac
 #endif
 
 	if (DUAL_MODE)
-		if (pool_read2(SECONDARY(backend), len) == NULL)
+	{
+		char *sp;
+
+		if ((sp = pool_read2(SECONDARY(backend), len_array[1]-4)) == NULL)
 			return POOL_END;
 
+		name = sp;
+		value = sp + strlen(name) + 1;
+
+		pool_debug("secondary name: %s value: %s", name, value);
+	}
+
 	return pool_write(frontend, p, len);
 
 }
@@ -2225,3 +2346,251 @@ void pool_send_error_message(POOL_CONNECTION *frontend, int protoMajor,
 	else
 		pool_error("send_error_message: unknown protocol major %d", protoMajor);
 }
+
+/*
+ * sends q query in sync manner.
+ * this function sends a query and wait for CommandComplete/ReadyForQuery.
+ * if an error occured, it returns with POOL_ERROR.
+ * this function does NOT handle SELECT/SHOW quries.
+ * if no_ready_for_query is non 0, returns without reading the packet
+ * length for ReadyForQuery. This mode is necessary when called from ReadyForQuery().
+ */
+static POOL_STATUS do_command(POOL_CONNECTION *backend, char *query, int protoMajor,
+							  int no_ready_for_query)
+{
+	int len;
+	int status;
+	char kind;
+	char *string;
+
+	pool_debug("do_command: Query: %s", query);
+
+	/* send the query to the backend */
+	pool_write(backend, "Q", 1);
+	len = strlen(query)+1;
+
+	if (protoMajor == PROTO_MAJOR_V3)
+	{
+		int sendlen = htonl(len + 4);
+		pool_write(backend, &sendlen, sizeof(sendlen));
+	}
+
+	if (pool_write_and_flush(backend, query, len) < 0)
+	{
+		return POOL_END;
+	}
+
+	/*
+	 * Expecting CompleteCommand
+	 */
+	status = pool_read(backend, &kind, sizeof(kind));
+	if (status < 0)
+	{
+		pool_error("do_command: error while reading message kind");
+		return POOL_END;
+	}
+
+#ifdef NOT_USED
+	if (kind != 'C')
+	{
+		pool_error("do_command: backend does not successfully complete command %s status %c", query, kind);
+
+		/* the response must be an ERROR. handle it */
+		
+		return POOL_END;
+	}
+#endif
+	/*
+	 * read command tag of CommandComplete response
+	 */
+	if (protoMajor == PROTO_MAJOR_V3)
+	{
+		if (pool_read(backend, &len, sizeof(len)) < 0)
+			return POOL_END;
+		len = ntohl(len) - 4;
+		string = pool_read2(backend, len);
+		if (string == NULL)
+			return POOL_END;
+		pool_debug("command tag: %s", string);
+	}
+	else
+	{
+		string = pool_read_string(backend, &len, 0);
+		if (string == NULL)
+			return POOL_END;
+	}
+
+	/*
+	 * Expecting ReadyForQuery
+	 */
+	status = pool_read(backend, &kind, sizeof(kind));
+	if (status < 0)
+	{
+		pool_error("do_command: error while reading message kind");
+		return POOL_END;
+	}
+
+	if (kind != 'Z')
+	{
+		pool_error("do_command: backend does not return ReadyForQuery");
+		return POOL_END;
+	}
+
+	if (no_ready_for_query)
+		return POOL_CONTINUE;
+
+	if (protoMajor == PROTO_MAJOR_V3)
+	{
+		if (pool_read(backend, &len, sizeof(len)) < 0)
+			return POOL_END;
+
+		status = pool_read(backend, &kind, sizeof(kind));
+		if (status < 0)
+		{
+			pool_error("do_command: error while reading transaction status");
+			return POOL_END;
+		}
+
+		/* set transaction state */
+		pool_debug("ReadyForQuery: transaction state: %c", kind);
+		backend->tstate = kind;
+	}
+
+	return POOL_CONTINUE;
+}
+
+/*
+ * judge if we need to lock the table
+ * to keep SERIAL consistency among servers
+ */
+static int need_insert_lock(POOL_CONNECTION_POOL *backend, char *query)
+{
+	if (MAJOR(backend) != PROTO_MAJOR_V3)
+		return 0;
+	
+	/*
+	 * either insert_lock directive specified and without "NO INSERT LOCK" comment
+	 * or "INSERT LOCK" comment exists?
+	 */
+	if ((pool_config.insert_lock && strncasecmp(query, NO_LOCK_COMMENT, NO_LOCK_COMMENT_SZ)) ||
+		strncasecmp(query, LOCK_COMMENT, LOCK_COMMENT_SZ) == 0)
+	{
+		/* INSERT STATEMENT? */
+		query = skip_comment(query);
+		if (strncasecmp(query, "INSERT", 6) == 0)
+			return 1;
+	}
+
+	return 0;
+}
+
+/*
+ * if a transaction has not already started, start a new one.
+ * issue LOCK TABLE IN SHARE ROW EXCLUSIVE MODE
+ */
+static POOL_STATUS insert_lock(POOL_CONNECTION_POOL *backend, char *query)
+{
+	char *table;
+	char qbuf[1024];
+	int i;
+
+	if (MAJOR(backend) != PROTO_MAJOR_V3)
+		return 0;
+
+	/* if we are not in a transaction block,
+	 * start a new transaction
+	 */
+	if (TSTATE(backend) == 'I')
+	{
+		for (i = 0;i < backend->num;i++)
+		{
+			if (do_command(backend->slots[i]->con, "BEGIN", PROTO_MAJOR_V3, 0) != POOL_CONTINUE)
+				return POOL_END;
+		}
+
+		/* mark that we started new transaction */
+		internal_transaction_started = 1;
+	}
+
+	/* issue lock table command */
+	table = get_insert_command_table_name(query);
+	snprintf(qbuf, sizeof(qbuf), "LOCK TABLE %s IN SHARE ROW EXCLUSIVE MODE", table);
+
+	for (i = 0;i < backend->num;i++)
+	{
+		if (do_command(backend->slots[i]->con, qbuf, PROTO_MAJOR_V3, 0) != POOL_CONTINUE)
+		{
+			internal_transaction_started = 0;
+			return POOL_END;
+		}
+	}
+
+	return POOL_CONTINUE;
+}
+
+/*
+ * obtain table name in INSERT statement
+ */
+static char *get_insert_command_table_name(char *query)
+{
+	static char table[1024];
+	char *qbuf;
+	char *token;
+
+	table[0] = '\0';
+
+	/* skip comment */
+    query = skip_comment(query);
+
+	if (*query == '\0')
+		return table;
+
+	/* skip spaces */
+	while (*query && isspace(*query))
+		query++;
+
+	/* skip non spaces(INSERT) */
+	while (*query && !isspace(*query))
+		query++;
+
+	/* skip spaces */
+	while (*query && isspace(*query))
+		query++;
+
+	/* skip non spaces(INTO) */
+	while (*query && !isspace(*query))
+		query++;
+
+	/* skip spaces */
+	while (*query && isspace(*query))
+		query++;
+
+	/* get table */
+	qbuf = strdup(query);
+	token = strtok(qbuf, " (");
+	strncpy(table, token, sizeof(table));
+	free(qbuf);
+
+	pool_debug("get_insert_command_table_name: extracted table name: %s", table);
+
+	return table;
+}
+
+/* skip SQL comments */
+static char *skip_comment(char *query)
+{
+	if (strncmp(query, "/*", 2) == 0)
+	{
+		query += 2;
+		while (query)
+		{
+			if (strncmp(query, "*/", 2) == 0)
+			{
+				query += 2;
+				break;
+			}
+			query++;
+		}
+	}
+	return query;
+}
