@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
-* $Header: /home/t-ishii/repository/pgpool/pool_stream.c,v 1.23 2004/09/02 02:58:40 t-ishii Exp $
+* $Header: /cvsroot/pgpool/pgpool/pool_stream.c,v 1.2 2005/06/13 13:29:37 t-ishii Exp $
 *
 * pgpool: a language independent connection pool server for PostgreSQL 
 * written by Tatsuo Ishii
 *
-* Copyright (c) 2003, 2004	Tatsuo Ishii
+* Copyright (c) 2003-2005	Tatsuo Ishii
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
@@ -141,7 +141,7 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 			if (cp->isbackend)
 			{
 			    /* fatal error, notice to parent and exit */
-			    notice_backend_error();
+			    notice_backend_error(!cp->issecondary_backend);
 			    exit(1);
 			}
 			else
@@ -156,7 +156,7 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 			if (cp->isbackend)
 			{
 			    /* fatal error, notice to parent and exit */
-			    notice_backend_error();
+			    notice_backend_error(!cp->issecondary_backend);
 				exit(1);
 			}
 			else
@@ -249,7 +249,7 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 			if (cp->isbackend)
 			{
 			    /* fatal error, notice to parent and exit */
-			    notice_backend_error();
+			    notice_backend_error(!cp->issecondary_backend);
 			    exit(1);
 			}
 			else
@@ -264,7 +264,7 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 			if (cp->isbackend)
 			{
 			    /* fatal error, notice to parent and exit */
-			    notice_backend_error();
+			    notice_backend_error(!cp->issecondary_backend);
 				exit(1);
 			}
 			else
@@ -300,18 +300,26 @@ int pool_write(POOL_CONNECTION *cp, void *buf, int len)
 */
 int pool_flush(POOL_CONNECTION *cp)
 {
-	if (fflush(cp->write_fd) != 0)
+	for (;;)
 	{
+		errno = 0;
+
+		if (fflush(cp->write_fd) == 0)
+			break;
+
+		if (errno == EAGAIN)
+			continue;
+
 		pool_error("pool_flush: fflush failed (%s)", strerror(errno));
 
 		if (cp->isbackend)
 		{
-		    notice_backend_error();
-		    exit(1);
+			notice_backend_error(!cp->issecondary_backend);
+			exit(1);
 		}
 		else
 		{
-		    return -1;
+			return -1;
 		}
 	}
 	return 0;
@@ -432,7 +440,7 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 
 			if (cp->isbackend)
 			{
-			    notice_backend_error();
+			    notice_backend_error(!cp->issecondary_backend);
 			    exit(1);
 			}
 			else
